@@ -8,6 +8,9 @@
 #define XGBOOST_R_H_ // NOLINT(*)
 
 
+#ifndef R_NO_REMAP
+#  define R_NO_REMAP
+#endif
 #include <R.h>
 #include <Rinternals.h>
 #include <R_ext/Altrep.h>
@@ -23,14 +26,6 @@
  */
 XGB_DLL SEXP XGCheckNullPtr_R(SEXP handle);
 
-/*!
- * \brief set the dimensions of an array in-place
- * \param arr
- * \param dims dimensions to set to the array
- * \return NULL value
- */
-XGB_DLL SEXP XGSetArrayDimInplace_R(SEXP arr, SEXP dims);
-
 /*!
  * \brief set the names of the dimensions of an array in-place
  * \param arr
@@ -39,6 +34,14 @@ XGB_DLL SEXP XGSetArrayDimInplace_R(SEXP arr, SEXP dims);
  */
 XGB_DLL SEXP XGSetArrayDimNamesInplace_R(SEXP arr, SEXP dim_names);
 
+/*!
+ * \brief set the names of a vector in-place
+ * \param arr
+ * \param names names for the dimensions to set
+ * \return NULL value
+ */
+XGB_DLL SEXP XGSetVectorNamesInplace_R(SEXP arr, SEXP names);
+
 /*!
  * \brief Set global configuration
  * \param json_str a JSON string representing the list of key-value pairs
