@@ -17,10 +17,16 @@ test_that("train and predict binary classification", {
   nrounds <- 2
   expect_output(
     bst <- xgb.train(
-      data = xgb.DMatrix(train$data, label = train$label), max_depth = 2,
-      eta = 1, nthread = n_threads, nrounds = nrounds,
-      objective = "binary:logistic", eval_metric = "error",
-      evals = list(train = xgb.DMatrix(train$data, label = train$label))
+      data = xgb.DMatrix(train$data, label = train$label, nthread = 1),
+      nrounds = nrounds,
+      params = xgb.params(
+        max_depth = 2,
+        learning_rate = 1,
+        nthread = n_threads,
+        objective = "binary:logistic",
+        eval_metric = "error"
+      ),
+      evals = list(train = xgb.DMatrix(train$data, label = train$label, nthread = 1))
     ),
     "train-error"
   )
@@ -103,15 +109,17 @@ test_that("dart prediction works", {
 
   set.seed(1994)
   booster_by_xgboost <- xgb.train(
-    data = xgb.DMatrix(d, label = y),
-    max_depth = 2,
-    booster = "dart",
-    rate_drop = 0.5,
-    one_drop = TRUE,
-    eta = 1,
-    nthread = n_threads,
+    data = xgb.DMatrix(d, label = y, nthread = 1),
     nrounds = nrounds,
-    objective = "reg:squarederror"
+    params = xgb.params(
+      max_depth = 2,
+      booster = "dart",
+      rate_drop = 0.5,
+      one_drop = TRUE,
+      learning_rate = 1,
+      nthread = n_threads,
+      objective = "reg:squarederror"
+    )
   )
   pred_by_xgboost_0 <- predict(booster_by_xgboost, newdata = d, iterationrange = NULL)
   pred_by_xgboost_1 <- predict(booster_by_xgboost, newdata = d, iterationrange = c(1, nrounds))
@@ -123,10 +131,10 @@ test_that("dart prediction works", {
   set.seed(1994)
   dtrain <- xgb.DMatrix(data = d, label = y, nthread = n_threads)
   booster_by_train <- xgb.train(
-    params = list(
+    params = xgb.params(
       booster = "dart",
       max_depth = 2,
-      eta = 1,
+      learning_rate = 1,
       rate_drop = 0.5,
       one_drop = TRUE,
       nthread = n_threads,
@@ -149,10 +157,13 @@ test_that("train and predict softprob", {
   set.seed(11)
   expect_output(
     bst <- xgb.train(
-      data = xgb.DMatrix(as.matrix(iris[, -5]), label = lb),
-      max_depth = 3, eta = 0.5, nthread = n_threads, nrounds = 5,
-      objective = "multi:softprob", num_class = 3, eval_metric = "merror",
-      evals = list(train = xgb.DMatrix(as.matrix(iris[, -5]), label = lb))
+      data = xgb.DMatrix(as.matrix(iris[, -5]), label = lb, nthread = 1),
+      nrounds = 5,
+      params = xgb.params(
+        max_depth = 3, learning_rate = 0.5, nthread = n_threads,
+        objective = "multi:softprob", num_class = 3, eval_metric = "merror"
+      ),
+      evals = list(train = xgb.DMatrix(as.matrix(iris[, -5]), label = lb, nthread = 1))
     ),
     "train-merror"
   )
@@ -162,20 +173,20 @@ test_that("train and predict softprob", {
   pred <- predict(bst, as.matrix(iris[, -5]))
   expect_length(pred, nrow(iris) * 3)
   # row sums add up to total probability of 1:
-  expect_equal(rowSums(matrix(pred, ncol = 3, byrow = TRUE)), rep(1, nrow(iris)), tolerance = 1e-7)
+  expect_equal(rowSums(pred), rep(1, nrow(iris)), tolerance = 1e-7)
   # manually calculate error at the last iteration:
-  mpred <- predict(bst, as.matrix(iris[, -5]), reshape = TRUE)
-  expect_equal(as.numeric(t(mpred)), pred)
+  mpred <- predict(bst, as.matrix(iris[, -5]))
+  expect_equal(mpred, pred)
   pred_labels <- max.col(mpred) - 1
   err <- sum(pred_labels != lb) / length(lb)
   expect_equal(attributes(bst)$evaluation_log[5, train_merror], err, tolerance = 5e-6)
   # manually calculate error at the 1st iteration:
-  mpred <- predict(bst, as.matrix(iris[, -5]), reshape = TRUE, iterationrange = c(1, 1))
+  mpred <- predict(bst, as.matrix(iris[, -5]), iterationrange = c(1, 1))
   pred_labels <- max.col(mpred) - 1
   err <- sum(pred_labels != lb) / length(lb)
   expect_equal(attributes(bst)$evaluation_log[1, train_merror], err, tolerance = 5e-6)
 
-  mpred1 <- predict(bst, as.matrix(iris[, -5]), reshape = TRUE, iterationrange = c(1, 1))
+  mpred1 <- predict(bst, as.matrix(iris[, -5]), iterationrange = c(1, 1))
   expect_equal(mpred, mpred1)
 
   d <- cbind(
@@ -186,11 +197,16 @@ test_that("train and predict softprob", {
   y <- sample.int(10, 100, replace = TRUE) - 1
   dtrain <- xgb.DMatrix(data = d, label = y, nthread = n_threads)
   booster <- xgb.train(
-    params = list(tree_method = "hist", nthread = n_threads),
-    data = dtrain, nrounds = 4, num_class = 10,
-    objective = "multi:softprob"
+    params = xgb.params(
+      objective = "multi:softprob",
+      num_class = 10,
+      tree_method = "hist",
+      nthread = n_threads
+    ),
+    data = dtrain,
+    nrounds = 4
   )
-  predt <- predict(booster, as.matrix(d), reshape = TRUE, strict_shape = FALSE)
+  predt <- predict(booster, as.matrix(d), strict_shape = FALSE)
   expect_equal(ncol(predt), 10)
   expect_equal(rowSums(predt), rep(1, 100), tolerance = 1e-7)
 })
@@ -200,10 +216,13 @@ test_that("train and predict softmax", {
   set.seed(11)
   expect_output(
     bst <- xgb.train(
-      data = xgb.DMatrix(as.matrix(iris[, -5]), label = lb),
-      max_depth = 3, eta = 0.5, nthread = n_threads, nrounds = 5,
-      objective = "multi:softmax", num_class = 3, eval_metric = "merror",
-      evals = list(train = xgb.DMatrix(as.matrix(iris[, -5]), label = lb))
+      data = xgb.DMatrix(as.matrix(iris[, -5]), label = lb, nthread = 1),
+      nrounds = 5,
+      params = xgb.params(
+        max_depth = 3, learning_rate = 0.5, nthread = n_threads,
+        objective = "multi:softmax", num_class = 3, eval_metric = "merror"
+      ),
+      evals = list(train = xgb.DMatrix(as.matrix(iris[, -5]), label = lb, nthread = 1))
     ),
     "train-merror"
   )
@@ -222,11 +241,16 @@ test_that("train and predict RF", {
   lb <- train$label
   # single iteration
   bst <- xgb.train(
-    data = xgb.DMatrix(train$data, label = lb), max_depth = 5,
-    nthread = n_threads,
-    nrounds = 1, objective = "binary:logistic", eval_metric = "error",
-    num_parallel_tree = 20, subsample = 0.6, colsample_bytree = 0.1,
-    evals = list(train = xgb.DMatrix(train$data, label = lb))
+    data = xgb.DMatrix(train$data, label = lb, nthread = 1),
+    nrounds = 1,
+    params = xgb.params(
+      max_depth = 5,
+      nthread = n_threads,
+      objective = "binary:logistic", eval_metric = "error",
+      num_parallel_tree = 20, subsample = 0.6, colsample_bytree = 0.1
+    ),
+    evals = list(train = xgb.DMatrix(train$data, label = lb, nthread = 1)),
+    verbose = 0
   )
   expect_equal(xgb.get.num.boosted.rounds(bst), 1)
 
@@ -245,22 +269,31 @@ test_that("train and predict RF with softprob", {
   nrounds <- 15
   set.seed(11)
   bst <- xgb.train(
-    data = xgb.DMatrix(as.matrix(iris[, -5]), label = lb),
-    max_depth = 3, eta = 0.9, nthread = n_threads, nrounds = nrounds,
-    objective = "multi:softprob", eval_metric = "merror",
-    num_class = 3, verbose = 0,
-    num_parallel_tree = 4, subsample = 0.5, colsample_bytree = 0.5,
-    evals = list(train = xgb.DMatrix(as.matrix(iris[, -5]), label = lb))
+    data = xgb.DMatrix(as.matrix(iris[, -5]), label = lb, nthread = 1),
+    nrounds = nrounds,
+    verbose = 0,
+    params = xgb.params(
+      max_depth = 3,
+      learning_rate = 0.9,
+      nthread = n_threads,
+      objective = "multi:softprob",
+      eval_metric = "merror",
+      num_class = 3,
+      num_parallel_tree = 4,
+      subsample = 0.5,
+      colsample_bytree = 0.5
+    ),
+    evals = list(train = xgb.DMatrix(as.matrix(iris[, -5]), label = lb, nthread = 1))
   )
   expect_equal(xgb.get.num.boosted.rounds(bst), 15)
   # predict for all iterations:
-  pred <- predict(bst, as.matrix(iris[, -5]), reshape = TRUE)
+  pred <- predict(bst, as.matrix(iris[, -5]))
   expect_equal(dim(pred), c(nrow(iris), 3))
   pred_labels <- max.col(pred) - 1
   err <- sum(pred_labels != lb) / length(lb)
   expect_equal(attributes(bst)$evaluation_log[nrounds, train_merror], err, tolerance = 5e-6)
   # predict for 7 iterations and adjust for 4 parallel trees per iteration
-  pred <- predict(bst, as.matrix(iris[, -5]), reshape = TRUE, iterationrange = c(1, 7))
+  pred <- predict(bst, as.matrix(iris[, -5]), iterationrange = c(1, 7))
   err <- sum((max.col(pred) - 1) != lb) / length(lb)
   expect_equal(attributes(bst)$evaluation_log[7, train_merror], err, tolerance = 5e-6)
 })
@@ -268,10 +301,14 @@ test_that("train and predict RF with softprob", {
 test_that("use of multiple eval metrics works", {
   expect_output(
     bst <- xgb.train(
-      data = xgb.DMatrix(train$data, label = train$label), max_depth = 2,
-      eta = 1, nthread = n_threads, nrounds = 2, objective = "binary:logistic",
-      eval_metric = "error", eval_metric = "auc", eval_metric = "logloss",
-      evals = list(train = xgb.DMatrix(train$data, label = train$label))
+      data = xgb.DMatrix(train$data, label = train$label, nthread = 1),
+      nrounds = 2,
+      params = list(
+        max_depth = 2,
+        learning_rate = 1, nthread = n_threads, objective = "binary:logistic",
+        eval_metric = "error", eval_metric = "auc", eval_metric = "logloss"
+      ),
+      evals = list(train = xgb.DMatrix(train$data, label = train$label, nthread = 1))
     ),
     "train-error.*train-auc.*train-logloss"
   )
@@ -280,10 +317,16 @@ test_that("use of multiple eval metrics works", {
   expect_equal(colnames(attributes(bst)$evaluation_log), c("iter", "train_error", "train_auc", "train_logloss"))
   expect_output(
     bst2 <- xgb.train(
-      data = xgb.DMatrix(train$data, label = train$label), max_depth = 2,
-      eta = 1, nthread = n_threads, nrounds = 2, objective = "binary:logistic",
-      eval_metric = list("error", "auc", "logloss"),
-      evals = list(train = xgb.DMatrix(train$data, label = train$label))
+      data = xgb.DMatrix(train$data, label = train$label, nthread = 1),
+      nrounds = 2,
+      params = xgb.params(
+        max_depth = 2,
+        learning_rate = 1,
+        nthread = n_threads,
+        objective = "binary:logistic",
+        eval_metric = list("error", "auc", "logloss")
+      ),
+      evals = list(train = xgb.DMatrix(train$data, label = train$label, nthread = 1))
     ),
     "train-error.*train-auc.*train-logloss"
   )
@@ -296,18 +339,18 @@ test_that("use of multiple eval metrics works", {
 test_that("training continuation works", {
   dtrain <- xgb.DMatrix(train$data, label = train$label, nthread = n_threads)
   evals <- list(train = dtrain)
-  param <- list(
-    objective = "binary:logistic", max_depth = 2, eta = 1, nthread = n_threads
+  params <- xgb.params(
+    objective = "binary:logistic", max_depth = 2, learning_rate = 1, nthread = n_threads
   )
 
   # for the reference, use 4 iterations at once:
   set.seed(11)
-  bst <- xgb.train(param, dtrain, nrounds = 4, evals = evals, verbose = 0)
+  bst <- xgb.train(params, dtrain, nrounds = 4, evals = evals, verbose = 0)
   # first two iterations:
   set.seed(11)
-  bst1 <- xgb.train(param, dtrain, nrounds = 2, evals = evals, verbose = 0)
+  bst1 <- xgb.train(params, dtrain, nrounds = 2, evals = evals, verbose = 0)
   # continue for two more:
-  bst2 <- xgb.train(param, dtrain, nrounds = 2, evals = evals, verbose = 0, xgb_model = bst1)
+  bst2 <- xgb.train(params, dtrain, nrounds = 2, evals = evals, verbose = 0, xgb_model = bst1)
   if (!windows_flag && !solaris_flag) {
     expect_equal(xgb.save.raw(bst), xgb.save.raw(bst2))
   }
@@ -315,7 +358,7 @@ test_that("training continuation works", {
   expect_equal(dim(attributes(bst2)$evaluation_log), c(4, 2))
   expect_equal(attributes(bst2)$evaluation_log, attributes(bst)$evaluation_log)
   # test continuing from raw model data
-  bst2 <- xgb.train(param, dtrain, nrounds = 2, evals = evals, verbose = 0, xgb_model = xgb.save.raw(bst1))
+  bst2 <- xgb.train(params, dtrain, nrounds = 2, evals = evals, verbose = 0, xgb_model = xgb.save.raw(bst1))
   if (!windows_flag && !solaris_flag) {
     expect_equal(xgb.save.raw(bst), xgb.save.raw(bst2))
   }
@@ -323,7 +366,7 @@ test_that("training continuation works", {
   # test continuing from a model in file
   fname <- file.path(tempdir(), "xgboost.json")
   xgb.save(bst1, fname)
-  bst2 <- xgb.train(param, dtrain, nrounds = 2, evals = evals, verbose = 0, xgb_model = fname)
+  bst2 <- xgb.train(params, dtrain, nrounds = 2, evals = evals, verbose = 0, xgb_model = fname)
   if (!windows_flag && !solaris_flag) {
     expect_equal(xgb.save.raw(bst), xgb.save.raw(bst2))
   }
@@ -334,9 +377,17 @@ test_that("xgb.cv works", {
   set.seed(11)
   expect_output(
     cv <- xgb.cv(
-      data = xgb.DMatrix(train$data, label = train$label), max_depth = 2, nfold = 5,
-      eta = 1., nthread = n_threads, nrounds = 2, objective = "binary:logistic",
-      eval_metric = "error", verbose = TRUE
+      data = xgb.DMatrix(train$data, label = train$label, nthread = 1),
+      nfold = 5,
+      nrounds = 2,
+      params = xgb.params(
+        max_depth = 2,
+        learning_rate = 1.,
+        nthread = n_threads,
+        objective = "binary:logistic",
+        eval_metric = "error"
+      ),
+      verbose = TRUE
     ),
     "train-error:"
   )
@@ -351,18 +402,64 @@ test_that("xgb.cv works", {
   expect_false(is.null(cv$call))
 })
 
+test_that("xgb.cv invalid inputs", {
+  data("mtcars")
+  y <- mtcars$mpg
+  x_df <- mtcars[, -1]
+
+  expect_error(
+    cv <- xgb.cv(
+      data = xgb.QuantileDMatrix(x_df, label = y),
+      nfold = 5,
+      nrounds = 2,
+      params = xgb.params(
+        max_depth = 2,
+        nthread = n_threads
+      )
+    ),
+    regexp = ".*QuantileDMatrix.*"
+  )
+  expect_error(
+    cv <- xgb.cv(
+      data = xgb.DMatrix(x_df, label = y),
+      nfold = 5,
+      nrounds = 2,
+      params = xgb.params(
+        max_depth = 2,
+        nthread = n_threads,
+      ),
+      callbacks = list(
+        xgb.cb.early.stop(stopping_rounds = 3, save_best = TRUE)
+      )
+    ),
+    regexp = ".*save_best.*"
+  )
+})
+
 test_that("xgb.cv works with stratified folds", {
   dtrain <- xgb.DMatrix(train$data, label = train$label, nthread = n_threads)
   set.seed(314159)
   cv <- xgb.cv(
-    data = dtrain, max_depth = 2, nfold = 5,
-    eta = 1., nthread = n_threads, nrounds = 2, objective = "binary:logistic",
+    data = dtrain,
+    nrounds = 2,
+    nfold = 5,
+    params = xgb.params(
+      max_depth = 2,
+      nthread = n_threads,
+      objective = "binary:logistic"
+    ),
     verbose = FALSE, stratified = FALSE
   )
   set.seed(314159)
   cv2 <- xgb.cv(
-    data = dtrain, max_depth = 2, nfold = 5,
-    eta = 1., nthread = n_threads, nrounds = 2, objective = "binary:logistic",
+    data = dtrain,
+    nfold = 5,
+    nrounds = 2,
+    params = xgb.params(
+      max_depth = 2,
+      nthread = n_threads,
+      objective = "binary:logistic"
+    ),
     verbose = FALSE, stratified = TRUE
   )
   # Stratified folds should result in a different evaluation logs
@@ -373,8 +470,13 @@ test_that("train and predict with non-strict classes", {
   # standard dense matrix input
   train_dense <- as.matrix(train$data)
   bst <- xgb.train(
-    data = xgb.DMatrix(train_dense, label = train$label), max_depth = 2,
-    eta = 1, nthread = n_threads, nrounds = 2, objective = "binary:logistic",
+    data = xgb.DMatrix(train_dense, label = train$label, nthread = 1),
+    nrounds = 2,
+    params = xgb.params(
+      max_depth = 2,
+      nthread = n_threads,
+      objective = "binary:logistic"
+    ),
     verbose = 0
   )
   pr0 <- predict(bst, train_dense)
@@ -384,8 +486,13 @@ test_that("train and predict with non-strict classes", {
   expect_true(is.matrix(train_dense))
   expect_error(
     bst <- xgb.train(
-      data = xgb.DMatrix(train_dense, label = train$label), max_depth = 2,
-      eta = 1, nthread = n_threads, nrounds = 2, objective = "binary:logistic",
+      data = xgb.DMatrix(train_dense, label = train$label, nthread = 1),
+      nrounds = 2,
+      params = xgb.params(
+        max_depth = 2,
+        nthread = n_threads,
+        objective = "binary:logistic"
+      ),
       verbose = 0
     ),
     regexp = NA
@@ -398,8 +505,13 @@ test_that("train and predict with non-strict classes", {
   expect_true(is.matrix(train_dense))
   expect_error(
     bst <- xgb.train(
-      data = xgb.DMatrix(train_dense, label = train$label), max_depth = 2,
-      eta = 1, nthread = n_threads, nrounds = 2, objective = "binary:logistic",
+      data = xgb.DMatrix(train_dense, label = train$label, nthread = 1),
+      nrounds = 2,
+      params = xgb.params(
+        max_depth = 2,
+        nthread = n_threads,
+        objective = "binary:logistic"
+      ),
       verbose = 0
     ),
     regexp = NA
@@ -418,16 +530,16 @@ test_that("max_delta_step works", {
     agaricus.train$data, label = agaricus.train$label, nthread = n_threads
   )
   evals <- list(train = dtrain)
-  param <- list(
+  params <- xgb.params(
     objective = "binary:logistic", eval_metric = "logloss", max_depth = 2,
     nthread = n_threads,
-    eta = 0.5
+    learning_rate = 0.5
   )
   nrounds <- 5
   # model with no restriction on max_delta_step
-  bst1 <- xgb.train(param, dtrain, nrounds, evals = evals, verbose = 1)
+  bst1 <- xgb.train(params, dtrain, nrounds, evals = evals, verbose = 0)
   # model with restricted max_delta_step
-  bst2 <- xgb.train(param, dtrain, nrounds, evals = evals, verbose = 1, max_delta_step = 1)
+  bst2 <- xgb.train(c(params, list(max_delta_step = 1)), dtrain, nrounds, evals = evals, verbose = 0)
   # the no-restriction model is expected to have consistently lower loss during the initial iterations
   expect_true(all(attributes(bst1)$evaluation_log$train_logloss < attributes(bst2)$evaluation_log$train_logloss))
   expect_lt(mean(attributes(bst1)$evaluation_log$train_logloss) / mean(attributes(bst2)$evaluation_log$train_logloss), 0.8)
@@ -447,13 +559,13 @@ test_that("colsample_bytree works", {
   evals <- list(train = dtrain, eval = dtest)
   ## Use colsample_bytree = 0.01, so that roughly one out of 100 features is chosen for
   ## each tree
-  param <- list(
-    max_depth = 2, eta = 0, nthread = n_threads,
+  params <- xgb.params(
+    max_depth = 2, learning_rate = 0, nthread = n_threads,
     colsample_bytree = 0.01, objective = "binary:logistic",
     eval_metric = "auc"
   )
   set.seed(2)
-  bst <- xgb.train(param, dtrain, nrounds = 100, evals = evals, verbose = 0)
+  bst <- xgb.train(params, dtrain, nrounds = 100, evals = evals, verbose = 0)
   xgb.importance(model = bst)
   # If colsample_bytree works properly, a variety of features should be used
   # in the 100 trees
@@ -462,9 +574,13 @@ test_that("colsample_bytree works", {
 
 test_that("Configuration works", {
   bst <- xgb.train(
-    data = xgb.DMatrix(train$data, label = train$label), max_depth = 2,
-    eta = 1, nthread = n_threads, nrounds = 2, objective = "binary:logistic",
-    eval_metric = "error", eval_metric = "auc", eval_metric = "logloss"
+    data = xgb.DMatrix(train$data, label = train$label, nthread = 1),
+    nrounds = 2,
+    params = xgb.params(
+      max_depth = 2,
+      nthread = n_threads,
+      objective = "binary:logistic"
+    )
   )
   config <- xgb.config(bst)
   xgb.config(bst) <- config
@@ -485,15 +601,25 @@ test_that("strict_shape works", {
     n_rows <- nrow(X)
     n_cols <- ncol(X)
 
-    expect_equal(dim(predt), c(n_groups, n_rows))
-    expect_equal(dim(margin), c(n_groups, n_rows))
-    expect_equal(dim(contri), c(n_cols + 1, n_groups, n_rows))
-    expect_equal(dim(interact), c(n_cols + 1, n_cols + 1, n_groups, n_rows))
-    expect_equal(dim(leaf), c(1, n_groups, n_rounds, n_rows))
+    expect_equal(dim(predt), c(n_rows, n_groups))
+    expect_equal(dim(margin), c(n_rows, n_groups))
+    expect_equal(dim(contri), c(n_rows, n_groups, n_cols + 1))
+    expect_equal(dim(interact), c(n_rows, n_groups, n_cols + 1, n_cols + 1))
+    expect_equal(dim(leaf), c(n_rows, n_rounds, n_groups, 1))
 
     if (n_groups != 1) {
       for (g in seq_len(n_groups)) {
-        expect_lt(max(abs(colSums(contri[, g, ]) - margin[g, ])), 1e-5)
+        expect_lt(max(abs(rowSums(contri[, g, ]) - margin[, g])), 1e-5)
+      }
+
+      leaf_no_strict <- predict(bst, X, strict_shape = FALSE, predleaf = TRUE)
+      for (g in seq_len(n_groups)) {
+        g_mask <- rep(FALSE, n_groups)
+        g_mask[g] <- TRUE
+        expect_equal(
+          leaf[, , g, 1L],
+          leaf_no_strict[, g_mask]
+        )
       }
     }
   }
@@ -503,9 +629,12 @@ test_that("strict_shape works", {
     X <- as.matrix(iris[, -5])
 
     bst <- xgb.train(
-      data = xgb.DMatrix(X, label = y),
-      max_depth = 2, nrounds = n_rounds, nthread = n_threads,
-      objective = "multi:softprob", num_class = 3, eval_metric = "merror"
+      data = xgb.DMatrix(X, label = y, nthread = 1),
+      nrounds = n_rounds,
+      params = xgb.params(
+        max_depth = 2, nthread = n_threads,
+        objective = "multi:softprob", num_class = 3
+      )
     )
 
     test_strict_shape(bst, X, 3)
@@ -518,9 +647,12 @@ test_that("strict_shape works", {
     y <- agaricus.train$label
 
     bst <- xgb.train(
-      data = xgb.DMatrix(X, label = y), max_depth = 2, nthread = n_threads,
-      nrounds = n_rounds, objective = "binary:logistic",
-      eval_metric = "error", eval_metric = "auc", eval_metric = "logloss"
+      data = xgb.DMatrix(X, label = y, nthread = 1),
+      nrounds = n_rounds,
+      params = xgb.params(
+        max_depth = 2, nthread = n_threads,
+        objective = "binary:logistic"
+      )
     )
 
     test_strict_shape(bst, X, 1)
@@ -537,8 +669,12 @@ test_that("'predict' accepts CSR data", {
   x_csr <- as(x_csc, "RsparseMatrix")
   x_spv <- as(x_csc, "sparseVector")
   bst <- xgb.train(
-    data = xgb.DMatrix(X, label = y), objective = "binary:logistic",
-    nrounds = 5L, verbose = FALSE, nthread = n_threads,
+    data = xgb.DMatrix(X, label = y, nthread = 1),
+    nrounds = 5L, verbose = FALSE,
+    params = xgb.params(
+      objective = "binary:logistic",
+      nthread = n_threads
+    )
   )
   p_csc <- predict(bst, x_csc)
   p_csr <- predict(bst, x_csr)
@@ -551,10 +687,10 @@ test_that("Quantile regression accepts multiple quantiles", {
   data(mtcars)
   y <- mtcars[, 1]
   x <- as.matrix(mtcars[, -1])
-  dm <- xgb.DMatrix(data = x, label = y)
+  dm <- xgb.DMatrix(data = x, label = y, nthread = 1)
   model <- xgb.train(
     data = dm,
-    params = list(
+    params = xgb.params(
       objective = "reg:quantileerror",
       tree_method = "exact",
       quantile_alpha = c(0.05, 0.5, 0.95),
@@ -562,7 +698,7 @@ test_that("Quantile regression accepts multiple quantiles", {
     ),
     nrounds = 15
   )
-  pred <- predict(model, x, reshape = TRUE)
+  pred <- predict(model, x)
 
   expect_equal(dim(pred)[1], nrow(x))
   expect_equal(dim(pred)[2], 3)
@@ -581,7 +717,7 @@ test_that("Can use multi-output labels with built-in objectives", {
   y_mirrored <- cbind(y, -y)
   dm <- xgb.DMatrix(x, label = y_mirrored, nthread = n_threads)
   model <- xgb.train(
-    params = list(
+    params = xgb.params(
       tree_method = "hist",
       multi_strategy = "multi_output_tree",
       objective = "reg:squarederror",
@@ -590,7 +726,7 @@ test_that("Can use multi-output labels with built-in objectives", {
     data = dm,
     nrounds = 5
   )
-  pred <- predict(model, x, reshape = TRUE)
+  pred <- predict(model, x)
   expect_equal(pred[, 1], -pred[, 2])
   expect_true(cor(y, pred[, 1]) > 0.9)
   expect_true(cor(y, pred[, 2]) < -0.9)
@@ -603,7 +739,7 @@ test_that("Can use multi-output labels with custom objectives", {
   y_mirrored <- cbind(y, -y)
   dm <- xgb.DMatrix(x, label = y_mirrored, nthread = n_threads)
   model <- xgb.train(
-    params = list(
+    params = xgb.params(
       tree_method = "hist",
       multi_strategy = "multi_output_tree",
       base_score = 0,
@@ -619,7 +755,7 @@ test_that("Can use multi-output labels with custom objectives", {
     data = dm,
     nrounds = 5
   )
-  pred <- predict(model, x, reshape = TRUE)
+  pred <- predict(model, x)
   expect_equal(pred[, 1], -pred[, 2])
   expect_true(cor(y, pred[, 1]) > 0.9)
   expect_true(cor(y, pred[, 2]) < -0.9)
@@ -632,14 +768,16 @@ test_that("Can use ranking objectives with either 'qid' or 'group'", {
   qid <- c(rep(1, 20), rep(2, 20), rep(3, 60))
   gr <- c(20, 20, 60)
 
-  dmat_qid <- xgb.DMatrix(x, label = y, qid = qid)
-  dmat_gr <- xgb.DMatrix(x, label = y, group = gr)
+  dmat_qid <- xgb.DMatrix(x, label = y, qid = qid, nthread = 1)
+  dmat_gr <- xgb.DMatrix(x, label = y, group = gr, nthread = 1)
 
-  params <- list(tree_method = "hist",
-                 lambdarank_num_pair_per_sample = 8,
-                 objective = "rank:ndcg",
-                 lambdarank_pair_method = "topk",
-                 nthread = n_threads)
+  params <- xgb.params(
+    tree_method = "hist",
+    lambdarank_num_pair_per_sample = 8,
+    objective = "rank:ndcg",
+    lambdarank_pair_method = "topk",
+    nthread = n_threads
+  )
   set.seed(123)
   model_qid <- xgb.train(params, dmat_qid, nrounds = 5)
   set.seed(123)
@@ -657,7 +795,7 @@ test_that("Can predict on data.frame objects", {
   x_mat <- as.matrix(x_df)
   dm <- xgb.DMatrix(x_mat, label = y, nthread = n_threads)
   model <- xgb.train(
-    params = list(
+    params = xgb.params(
       tree_method = "hist",
       objective = "reg:squarederror",
       nthread = n_threads
@@ -666,9 +804,9 @@ test_that("Can predict on data.frame objects", {
     nrounds = 5
   )
 
-  pred_mat <- predict(model, xgb.DMatrix(x_mat), nthread = n_threads)
-  pred_df <- predict(model, x_df, nthread = n_threads)
-  expect_equal(pred_mat, pred_df)
+  pred_mat <- predict(model, xgb.DMatrix(x_mat, nthread = 1))
+  pred_df <- predict(model, x_df)
+  expect_equal(pred_mat, unname(pred_df))
 })
 
 test_that("'base_margin' gives the same result in DMatrix as in inplace_predict", {
@@ -677,7 +815,7 @@ test_that("'base_margin' gives the same result in DMatrix as in inplace_predict"
   x <- as.matrix(mtcars[, -1])
   dm <- xgb.DMatrix(x, label = y, nthread = n_threads)
   model <- xgb.train(
-    params = list(
+    params = xgb.params(
       tree_method = "hist",
       objective = "reg:squarederror",
       nthread = n_threads
@@ -688,11 +826,11 @@ test_that("'base_margin' gives the same result in DMatrix as in inplace_predict"
 
   set.seed(123)
   base_margin <- rnorm(nrow(x))
-  dm_w_base <- xgb.DMatrix(data = x, base_margin = base_margin)
+  dm_w_base <- xgb.DMatrix(data = x, base_margin = base_margin, nthread = 1)
   pred_from_dm <- predict(model, dm_w_base)
   pred_from_mat <- predict(model, x, base_margin = base_margin)
 
-  expect_equal(pred_from_dm, pred_from_mat)
+  expect_equal(pred_from_dm, unname(pred_from_mat))
 })
 
 test_that("Coefficients from gblinear have the expected shape and names", {
@@ -704,7 +842,7 @@ test_that("Coefficients from gblinear have the expected shape and names", {
   dm <- xgb.DMatrix(x, label = y, nthread = 1)
   model <- xgb.train(
     data = dm,
-    params = list(
+    params = xgb.params(
       booster = "gblinear",
       nthread = 1
     ),
@@ -715,7 +853,7 @@ test_that("Coefficients from gblinear have the expected shape and names", {
   expect_equal(names(coefs), c("(Intercept)", colnames(x)))
   pred_auto <- predict(model, x)
   pred_manual <- as.numeric(mm %*% coefs)
-  expect_equal(pred_manual, pred_auto, tolerance = 1e-5)
+  expect_equal(pred_manual, unname(pred_auto), tolerance = 1e-5)
 
   # Multi-column coefficients
   data(iris)
@@ -725,7 +863,7 @@ test_that("Coefficients from gblinear have the expected shape and names", {
   mm <- model.matrix(~., data = iris[, -5])
   model <- xgb.train(
     data = dm,
-    params = list(
+    params = xgb.params(
       booster = "gblinear",
       objective = "multi:softprob",
       num_class = 3,
@@ -737,9 +875,22 @@ test_that("Coefficients from gblinear have the expected shape and names", {
   expect_equal(nrow(coefs), ncol(x) + 1)
   expect_equal(ncol(coefs), 3)
   expect_equal(row.names(coefs), c("(Intercept)", colnames(x)))
-  pred_auto <- predict(model, x, outputmargin = TRUE, reshape = TRUE)
+  pred_auto <- predict(model, x, outputmargin = TRUE)
   pred_manual <- unname(mm %*% coefs)
   expect_equal(pred_manual, pred_auto, tolerance = 1e-7)
+
+  # xgboost() with additional metadata
+  model <- xgboost(
+    iris[, -5],
+    iris$Species,
+    booster = "gblinear",
+    objective = "multi:softprob",
+    nrounds = 3,
+    nthread = 1
+  )
+  coefs <- coef(model)
+  expect_equal(row.names(coefs), c("(Intercept)", colnames(x)))
+  expect_equal(colnames(coefs), levels(iris$Species))
 })
 
 test_that("Deep copies work as expected", {
@@ -749,7 +900,7 @@ test_that("Deep copies work as expected", {
   dm <- xgb.DMatrix(x, label = y, nthread = 1)
   model <- xgb.train(
    data = dm,
-   params = list(nthread = 1),
+   params = xgb.params(nthread = 1),
    nrounds = 3
   )
 
@@ -772,7 +923,7 @@ test_that("Pointer comparison works as expected", {
   y <- mtcars$mpg
   x <- as.matrix(mtcars[, -1])
   model <- xgb.train(
-    params = list(nthread = 1),
+    params = xgb.params(nthread = 1),
     data = xgb.DMatrix(x, label = y, nthread = 1),
     nrounds = 3
   )
@@ -796,12 +947,12 @@ test_that("DMatrix field are set to booster when training", {
 
   dm_unnamed <- xgb.DMatrix(x, label = y, nthread = 1)
   dm_feature_names <- xgb.DMatrix(x, label = y, feature_names = c("a", "b", "c"), nthread = 1)
-  dm_feature_types <- xgb.DMatrix(x, label = y)
+  dm_feature_types <- xgb.DMatrix(x, label = y, nthread = 1)
   setinfo(dm_feature_types, "feature_type", c("q", "c", "q"))
   dm_both <- xgb.DMatrix(x, label = y, feature_names = c("a", "b", "c"), nthread = 1)
   setinfo(dm_both, "feature_type", c("q", "c", "q"))
 
-  params <- list(nthread = 1)
+  params <- xgb.params(nthread = 1)
   model_unnamed <- xgb.train(data = dm_unnamed, params = params, nrounds = 3)
   model_feature_names <- xgb.train(data = dm_feature_names, params = params, nrounds = 3)
   model_feature_types <- xgb.train(data = dm_feature_types, params = params, nrounds = 3)
@@ -830,7 +981,7 @@ test_that("Seed in params override PRNG from R", {
       agaricus.train$data,
       label = agaricus.train$label, nthread = 1L
     ),
-    params = list(
+    params = xgb.params(
       objective = "binary:logistic",
       max_depth = 3L,
       subsample = 0.1,
@@ -846,7 +997,7 @@ test_that("Seed in params override PRNG from R", {
       agaricus.train$data,
       label = agaricus.train$label, nthread = 1L
     ),
-    params = list(
+    params = xgb.params(
       objective = "binary:logistic",
       max_depth = 3L,
       subsample = 0.1,
@@ -867,7 +1018,7 @@ test_that("Seed in params override PRNG from R", {
       agaricus.train$data,
       label = agaricus.train$label, nthread = 1L
     ),
-    params = list(
+    params = xgb.params(
       objective = "binary:logistic",
       max_depth = 3L,
       subsample = 0.1,
@@ -890,7 +1041,7 @@ test_that("xgb.cv works for AFT", {
   X <- matrix(c(1, -1, -1, 1, 0, 1, 1, 0), nrow = 4, byrow = TRUE)  # 4x2 matrix
   dtrain <- xgb.DMatrix(X, nthread = n_threads)
 
-  params <- list(objective = 'survival:aft', learning_rate = 0.2, max_depth = 2L)
+  params <- xgb.params(objective = 'survival:aft', learning_rate = 0.2, max_depth = 2L, nthread = n_threads)
 
   # data must have bounds
   expect_error(
@@ -898,8 +1049,7 @@ test_that("xgb.cv works for AFT", {
       params = params,
       data = dtrain,
       nround = 5L,
-      nfold = 4L,
-      nthread = n_threads
+      nfold = 4L
     )
   )
 
@@ -910,7 +1060,7 @@ test_that("xgb.cv works for AFT", {
   expect_warning(
     xgb.cv(
       params = params, data = dtrain, nround = 5L, nfold = 4L,
-      nthread = n_threads, stratified = TRUE, verbose = FALSE
+      stratified = TRUE, verbose = FALSE
     )
   )
 
@@ -925,12 +1075,13 @@ test_that("xgb.cv works for ranking", {
   x <- iris[, -(4:5)]
   y <- as.integer(iris$Petal.Width)
   group <- rep(50, 3)
-  dm <- xgb.DMatrix(x, label = y, group = group)
+  dm <- xgb.DMatrix(x, label = y, group = group, nthread = 1)
   res <- xgb.cv(
     data = dm,
-    params = list(
+    params = xgb.params(
       objective = "rank:pairwise",
-      max_depth = 3
+      max_depth = 3,
+      nthread = 1L
     ),
     nrounds = 3,
     nfold = 2,
@@ -939,3 +1090,47 @@ test_that("xgb.cv works for ranking", {
   )
   expect_equal(length(res$folds), 2L)
 })
+
+test_that("Row names are preserved in outputs", {
+  data(iris)
+  x <- iris[, -5]
+  y <- as.numeric(iris$Species) - 1
+  dm <- xgb.DMatrix(x, label = y, nthread = 1)
+  model <- xgb.train(
+    data = dm,
+    params = xgb.params(
+      objective = "multi:softprob",
+      num_class = 3,
+      max_depth = 2,
+      nthread = 1
+    ),
+    nrounds = 3
+  )
+  row.names(x) <- paste0("r", seq(1, nrow(x)))
+  pred <- predict(model, x)
+  expect_equal(row.names(pred), row.names(x))
+  pred <- predict(model, x, avoid_transpose = TRUE)
+  expect_equal(colnames(pred), row.names(x))
+
+  data(mtcars)
+  y <- mtcars[, 1]
+  x <- as.matrix(mtcars[, -1])
+  dm <- xgb.DMatrix(data = x, label = y, nthread = 1)
+  model <- xgb.train(
+    data = dm,
+    params = xgb.params(
+      max_depth = 2,
+      nthread = 1
+    ),
+    nrounds = 3
+  )
+  row.names(x) <- paste0("r", seq(1, nrow(x)))
+  pred <- predict(model, x)
+  expect_equal(names(pred), row.names(x))
+  pred <- predict(model, x, avoid_transpose = TRUE)
+  expect_equal(names(pred), row.names(x))
+  pred <- predict(model, x, predleaf = TRUE)
+  expect_equal(row.names(pred), row.names(x))
+  pred <- predict(model, x, predleaf = TRUE, avoid_transpose = TRUE)
+  expect_equal(colnames(pred), row.names(x))
+})
