@@ -41,13 +41,13 @@ test_that("xgb.DMatrix: basic construction", {
 
   params <- list(tree_method = "hist", nthread = n_threads)
   bst_fd <- xgb.train(
-    params, nrounds = 8, fd, evals = list(train = fd)
+    params, nrounds = 8, fd, evals = list(train = fd), verbose = 0
   )
   bst_dgr <- xgb.train(
-    params, nrounds = 8, fdgr, evals = list(train = fdgr)
+    params, nrounds = 8, fdgr, evals = list(train = fdgr), verbose = 0
   )
   bst_dgc <- xgb.train(
-    params, nrounds = 8, fdgc, evals = list(train = fdgc)
+    params, nrounds = 8, fdgc, evals = list(train = fdgc), verbose = 0
   )
 
   raw_fd <- xgb.save.raw(bst_fd, raw_format = "ubj")
@@ -103,8 +103,10 @@ test_that("xgb.DMatrix: saving, loading", {
   on.exit(unlink(tmp_file))
   expect_true(xgb.DMatrix.save(dtest1, tmp_file))
   # read from a local file
-  expect_output(dtest3 <- xgb.DMatrix(tmp_file), "entries loaded from")
-  expect_output(dtest3 <- xgb.DMatrix(tmp_file, silent = TRUE), NA)
+  xgb.set.config(verbosity = 2)
+  expect_output(dtest3 <- xgb.DMatrix(tmp_file, nthread = 1), "entries loaded from")
+  xgb.set.config(verbosity = 1)
+  expect_output(dtest3 <- xgb.DMatrix(tmp_file, nthread = 1), NA)
   unlink(tmp_file)
   expect_equal(getinfo(dtest1, 'label'), getinfo(dtest3, 'label'))
 
@@ -128,7 +130,8 @@ test_that("xgb.DMatrix: saving, loading", {
   expect_equal(length(cnames), 126)
   tmp_file <- tempfile('xgb.DMatrix_')
   xgb.DMatrix.save(dtrain, tmp_file)
-  dtrain <- xgb.DMatrix(tmp_file)
+  xgb.set.config(verbosity = 0)
+  dtrain <- xgb.DMatrix(tmp_file, nthread = 1)
   expect_equal(colnames(dtrain), cnames)
 
   ft <- rep(c("c", "q"), each = length(cnames) / 2)
@@ -347,8 +350,8 @@ test_that("xgb.DMatrix: can get group for both 'qid' and 'group' constructors",
   group <- c(20, 20, 60)
   qid <- c(rep(1, 20), rep(2, 20), rep(3, 60))
 
-  gr_mat <- xgb.DMatrix(x, group = group)
-  qid_mat <- xgb.DMatrix(x, qid = qid)
+  gr_mat <- xgb.DMatrix(x, group = group, nthread = 1)
+  qid_mat <- xgb.DMatrix(x, qid = qid, nthread = 1)
 
   info_gr <- getinfo(gr_mat, "group")
   info_qid <- getinfo(qid_mat, "group")
@@ -369,7 +372,7 @@ test_that("xgb.DMatrix: data.frame", {
     stringsAsFactors = TRUE
   )
 
-  m <- xgb.DMatrix(df)
+  m <- xgb.DMatrix(df, nthread = 1)
   expect_equal(colnames(m), colnames(df))
   expect_equal(
     getinfo(m, "feature_type"), c("float", "float", "int", "i", "c", "c")
@@ -380,7 +383,7 @@ test_that("xgb.DMatrix: data.frame", {
     valid = c("a", "b", "d", "c"),
     stringsAsFactors = TRUE
   )
-  m <- xgb.DMatrix(df)
+  m <- xgb.DMatrix(df, nthread = 1)
   expect_equal(getinfo(m, "feature_type"), c("c", "c"))
 })
 
@@ -400,12 +403,10 @@ test_that("xgb.DMatrix: can take multi-dimensional 'base_margin'", {
     ),
     nround = 1
   )
-  pred_only_x <- predict(model, x, nthread = n_threads, reshape = TRUE)
+  pred_only_x <- predict(model, x)
   pred_w_base <- predict(
     model,
-    xgb.DMatrix(data = x, base_margin = b, nthread = n_threads),
-    nthread = n_threads,
-    reshape = TRUE
+    xgb.DMatrix(data = x, base_margin = b, nthread = 1)
   )
   expect_equal(pred_only_x, pred_w_base - b, tolerance = 1e-5)
 })
@@ -480,7 +481,7 @@ test_that("xgb.DMatrix: QuantileDMatrix is not accepted by exact method", {
   })
 })
 
-test_that("xgb.DMatrix: ExternalDMatrix produces the same results as regular DMatrix", {
+test_that("xgb.DMatrix: ExtMemDMatrix produces the same results as regular DMatrix", {
   data(mtcars)
   y <- mtcars[, 1]
   x <- as.matrix(mtcars[, -1])
@@ -490,11 +491,12 @@ test_that("xgb.DMatrix: ExternalDMatrix produces the same results as regular DMa
     nthread = n_threads
   )
   model <- xgb.train(
-    data = xgb.DMatrix(x, label = y),
+    data = xgb.DMatrix(x, label = y, nthread = 1),
     params = params,
     nrounds = 5
   )
   pred <- predict(model, x)
+  pred <- unname(pred)
 
   iterator_env <- as.environment(
     list(
@@ -529,8 +531,8 @@ test_that("xgb.DMatrix: ExternalDMatrix produces the same results as regular DMa
     f_reset = iterator_reset
   )
   cache_prefix <- tempdir()
-  edm <- xgb.ExternalDMatrix(data_iterator, cache_prefix, nthread = 1)
-  expect_true(inherits(edm, "xgb.ExternalDMatrix"))
+  edm <- xgb.ExtMemDMatrix(data_iterator, cache_prefix, nthread = 1)
+  expect_true(inherits(edm, "xgb.ExtMemDMatrix"))
   expect_true(inherits(edm, "xgb.DMatrix"))
   set.seed(123)
   model_ext <- xgb.train(
@@ -540,7 +542,7 @@ test_that("xgb.DMatrix: ExternalDMatrix produces the same results as regular DMa
   )
 
   pred_model1_edm <- predict(model, edm)
-  pred_model2_mat <- predict(model_ext, x)
+  pred_model2_mat <- predict(model_ext, x) |> unname()
   pred_model2_edm <- predict(model_ext, edm)
 
   expect_equal(pred_model1_edm, pred)
@@ -569,6 +571,7 @@ test_that("xgb.DMatrix: External QDM produces same results as regular QDM", {
     nrounds = 5
   )
   pred <- predict(model, x)
+  pred <- unname(pred)
 
   iterator_env <- as.environment(
     list(
@@ -618,7 +621,7 @@ test_that("xgb.DMatrix: External QDM produces same results as regular QDM", {
   )
 
   pred_model1_qdm <- predict(model, qdm)
-  pred_model2_mat <- predict(model_ext, x)
+  pred_model2_mat <- predict(model_ext, x) |> unname()
   pred_model2_qdm <- predict(model_ext, qdm)
 
   expect_equal(pred_model1_qdm, pred)
@@ -660,25 +663,25 @@ test_that("xgb.DMatrix: R errors thrown on DataIterator are thrown back to the u
     f_reset = iterator_reset
   )
   expect_error(
-    {xgb.ExternalDMatrix(data_iterator, nthread = 1)},
+    {xgb.ExtMemDMatrix(data_iterator, nthread = 1)},
     "custom error"
   )
 })
 
 test_that("xgb.DMatrix: number of non-missing matches data", {
   x <- matrix(1:10, nrow = 5)
-  dm1 <- xgb.DMatrix(x)
+  dm1 <- xgb.DMatrix(x, nthread = 1)
   expect_equal(xgb.get.DMatrix.num.non.missing(dm1), 10)
 
   x[2, 2] <- NA
   x[4, 1] <- NA
-  dm2 <- xgb.DMatrix(x)
+  dm2 <- xgb.DMatrix(x, nthread = 1)
   expect_equal(xgb.get.DMatrix.num.non.missing(dm2), 8)
 })
 
 test_that("xgb.DMatrix: retrieving data as CSR", {
   data(mtcars)
-  dm <- xgb.DMatrix(as.matrix(mtcars))
+  dm <- xgb.DMatrix(as.matrix(mtcars), nthread = 1)
   csr <- xgb.get.DMatrix.data(dm)
   expect_equal(dim(csr), dim(mtcars))
   expect_equal(colnames(csr), colnames(mtcars))
@@ -689,7 +692,7 @@ test_that("xgb.DMatrix: quantile cuts look correct", {
   data(mtcars)
   y <- mtcars$mpg
   x <- as.matrix(mtcars[, -1])
-  dm <- xgb.DMatrix(x, label = y)
+  dm <- xgb.DMatrix(x, label = y, nthread = 1)
   model <- xgb.train(
     data = dm,
     params = list(
@@ -771,7 +774,7 @@ test_that("xgb.DMatrix: can read CSV", {
   fname <- file.path(tempdir(), "data.csv")
   writeChar(txt, fname)
   uri <- paste0(fname, "?format=csv&label_column=0")
-  dm <- xgb.DMatrix(uri, silent = TRUE)
+  dm <- xgb.DMatrix(uri, silent = TRUE, nthread = 1)
   expect_equal(getinfo(dm, "label"), c(1, -1))
   expect_equal(
     as.matrix(xgb.get.DMatrix.data(dm)),
