@@ -21,8 +21,12 @@ test_that("predict feature interactions works", {
   y <- f_int(X)
 
   dm <- xgb.DMatrix(X, label = y, nthread = n_threads)
-  param <- list(
-    eta = 0.1, max_depth = 4, base_score = mean(y), lambda = 0, nthread = n_threads
+  param <- xgb.params(
+    learning_rate = 0.1,
+    max_depth = 4,
+    base_score = mean(y),
+    reg_lambda = 0,
+    nthread = n_threads
   )
   b <- xgb.train(param, dm, 100)
 
@@ -48,7 +52,7 @@ test_that("predict feature interactions works", {
   intr <- predict(b, dm, predinteraction = TRUE)
   expect_equal(dim(intr), c(N, P + 1, P + 1))
   # check assigned colnames
-  cn <- c(letters[1:P], "BIAS")
+  cn <- c(letters[1:P], "(Intercept)")
   expect_equal(dimnames(intr), list(NULL, cn, cn))
 
   # check the symmetry
@@ -60,7 +64,7 @@ test_that("predict feature interactions works", {
   # diagonal terms for features 3,4,5 must be close to zero
   expect_lt(Reduce(max, sapply(3:P, function(i) max(abs(intr[, i, i])))), 0.05)
 
-  # BIAS must have no interactions
+  # Intercept must have no interactions
   expect_lt(max(abs(intr[, 1:P, P + 1])), 0.00001)
 
   # interactions other than 2 x 3 must be close to zero
@@ -105,7 +109,7 @@ test_that("SHAP contribution values are not NAN", {
       eval_metric = "rmse",
       nthread = n_threads
     ),
-    data = xgb.DMatrix(as.matrix(subset(d, fold == 2)[, ivs]), label = subset(d, fold == 2)$y),
+    data = xgb.DMatrix(as.matrix(subset(d, fold == 2)[, ivs]), label = subset(d, fold == 2)$y, nthread = 1),
     nrounds = 3
   )
 
@@ -123,45 +127,31 @@ test_that("multiclass feature interactions work", {
   dm <- xgb.DMatrix(
     as.matrix(iris[, -5]), label = as.numeric(iris$Species) - 1, nthread = n_threads
   )
-  param <- list(
-    eta = 0.1, max_depth = 4, objective = 'multi:softprob', num_class = 3, nthread = n_threads
+  param <- xgb.params(
+    learning_rate = 0.1,
+    max_depth = 4,
+    objective = 'multi:softprob',
+    num_class = 3,
+    nthread = n_threads
   )
   b <- xgb.train(param, dm, 40)
-  pred <- t(
-    array(
-      data = predict(b, dm, outputmargin = TRUE),
-      dim = c(3, 150)
-    )
-  )
+  pred <- predict(b, dm, outputmargin = TRUE)
 
   # SHAP contributions:
   cont <- predict(b, dm, predcontrib = TRUE)
-  expect_length(cont, 3)
-  # rewrap them as a 3d array
-  cont <- array(
-    data = unlist(cont),
-    dim = c(150, 5,  3)
-  )
+  expect_length(dim(cont), 3)
 
   # make sure for each row they add up to marginal predictions
-  expect_lt(max(abs(apply(cont, c(1, 3), sum) - pred)), 0.001)
+  expect_lt(max(abs(apply(cont, c(1, 2), sum) - pred)), 0.001)
 
   # SHAP interaction contributions:
   intr <- predict(b, dm, predinteraction = TRUE)
-  expect_length(intr, 3)
-  # rewrap them as a 4d array
-  intr <- aperm(
-    a = array(
-      data = unlist(intr),
-      dim = c(150, 5, 5, 3)
-    ),
-    perm = c(4, 1, 2, 3)  # [grp, row, col, col]
-  )
+  expect_length(dim(intr), 4)
 
   # check the symmetry
   expect_lt(max(abs(aperm(intr, c(1, 2, 4, 3)) - intr)), 0.00001)
   # sums WRT columns must be close to feature contributions
-  expect_lt(max(abs(apply(intr, c(1, 2, 3), sum) - aperm(cont, c(3, 1, 2)))), 0.00001)
+  expect_lt(max(abs(apply(intr, c(1, 2, 3), sum) - cont)), 0.00001)
 })
 
 
@@ -169,11 +159,13 @@ test_that("SHAP single sample works", {
   train <- agaricus.train
   test <- agaricus.test
   booster <- xgb.train(
-    data = xgb.DMatrix(train$data, label = train$label),
-    max_depth = 2,
+    data = xgb.DMatrix(train$data, label = train$label, nthread = 1),
     nrounds = 4,
-    objective = "binary:logistic",
-    nthread = n_threads
+    params = xgb.params(
+      max_depth = 2,
+      objective = "binary:logistic",
+      nthread = n_threads
+    )
   )
 
   predt <- predict(
