@@ -2,16 +2,20 @@
 Example of using callbacks with Dask
 ====================================
 """
+
+from typing import Any
+
 import numpy as np
 from dask.distributed import Client, LocalCluster
 from dask_ml.datasets import make_regression
 from dask_ml.model_selection import train_test_split
 
 import xgboost as xgb
+import xgboost.dask as dxgb
 from xgboost.dask import DaskDMatrix
 
 
-def probability_for_going_backward(epoch):
+def probability_for_going_backward(epoch: int) -> float:
     return 0.999 / (1.0 + 0.05 * np.log(1.0 + epoch))
 
 
@@ -21,7 +25,9 @@ class CustomEarlyStopping(xgb.callback.TrainingCallback):
     In the beginning, allow the metric to become worse with a probability of 0.999.
     As boosting progresses, the probability should be adjusted downward"""
 
-    def __init__(self, *, validation_set, target_metric, maximize, seed):
+    def __init__(
+        self, *, validation_set: str, target_metric: str, maximize: bool, seed: int
+    ) -> None:
         self.validation_set = validation_set
         self.target_metric = target_metric
         self.maximize = maximize
@@ -32,7 +38,9 @@ class CustomEarlyStopping(xgb.callback.TrainingCallback):
         else:
             self.better = lambda x, y: x < y
 
-    def after_iteration(self, model, epoch, evals_log):
+    def after_iteration(
+        self, model: Any, epoch: int, evals_log: xgb.callback.TrainingCallback.EvalsLog
+    ) -> bool:
         metric_history = evals_log[self.validation_set][self.target_metric]
         if len(metric_history) < 2 or self.better(
             metric_history[-1], metric_history[-2]
@@ -40,7 +48,7 @@ class CustomEarlyStopping(xgb.callback.TrainingCallback):
             return False  # continue training
         p = probability_for_going_backward(epoch)
         go_backward = self.rng.choice(2, size=(1,), replace=True, p=[1 - p, p]).astype(
-            np.bool
+            np.bool_
         )[0]
         print(
             "The validation metric went into the wrong direction. "
@@ -52,7 +60,7 @@ class CustomEarlyStopping(xgb.callback.TrainingCallback):
             return True  # stop training
 
 
-def main(client):
+def main(client: Client) -> None:
     m = 100000
     n = 100
     X, y = make_regression(n_samples=m, n_features=n, chunks=200, random_state=0)
@@ -61,7 +69,7 @@ def main(client):
     dtrain = DaskDMatrix(client, X_train, y_train)
     dtest = DaskDMatrix(client, X_test, y_test)
 
-    output = xgb.dask.train(
+    output = dxgb.train(
         client,
         {
             "verbosity": 1,
