@@ -1,5 +1,5 @@
 /**
- * Copyright 2020-2023, XGBoost Contributors
+ * Copyright 2020-2025, XGBoost Contributors
  * \file global_config.h
  * \brief Global configuration for XGBoost
  * \author Hyunsu Cho
@@ -16,6 +16,8 @@ namespace xgboost {
 struct GlobalConfiguration : public XGBoostParameter<GlobalConfiguration> {
   std::int32_t verbosity{1};
   bool use_rmm{false};
+  // This is not a dmlc parameter to avoid conflict with the context class.
+  std::int32_t nthread{0};
   DMLC_DECLARE_PARAMETER(GlobalConfiguration) {
     DMLC_DECLARE_FIELD(verbosity)
         .set_range(0, 3)
@@ -27,6 +29,14 @@ struct GlobalConfiguration : public XGBoostParameter<GlobalConfiguration> {
 };
 
 using GlobalConfigThreadLocalStore = dmlc::ThreadLocalStore<GlobalConfiguration>;
+
+struct InitNewThread {
+  GlobalConfiguration config;
+  std::int32_t device{-1};
+
+  void operator()() const;
+  InitNewThread();
+};
 }  // namespace xgboost
 
 #endif  // XGBOOST_GLOBAL_CONFIG_H_
