@@ -11,6 +11,11 @@ class XGBoostLibraryNotFound(Exception):
     """Error thrown by when xgboost is not found"""
 
 
+def is_sphinx_build() -> bool:
+    """`XGBOOST_BUILD_DOC` is used by the sphinx conf.py to skip building the C++ code."""
+    return bool(os.environ.get("XGBOOST_BUILD_DOC", False))
+
+
 def find_lib_path() -> List[str]:
     """Find the path to xgboost dynamic library files.
 
@@ -38,6 +43,9 @@ def find_lib_path() -> List[str]:
                 os.path.join(sys.base_prefix, "Library"),
                 os.path.join(sys.base_prefix, "Library", "bin"),
                 os.path.join(sys.base_prefix, "Library", "lib"),
+                os.path.join(sys.base_prefix, "Library", "mingw-w64"),
+                os.path.join(sys.base_prefix, "Library", "mingw-w64", "bin"),
+                os.path.join(sys.base_prefix, "Library", "mingw-w64", "lib"),
             ]
         )
         dll_path = [os.path.join(p, "xgboost.dll") for p in dll_path]
@@ -52,8 +60,7 @@ def find_lib_path() -> List[str]:
 
     lib_path = [p for p in dll_path if os.path.exists(p) and os.path.isfile(p)]
 
-    # XGBOOST_BUILD_DOC is defined by sphinx conf.
-    if not lib_path and not os.environ.get("XGBOOST_BUILD_DOC", False):
+    if not lib_path and not is_sphinx_build():
         link = "https://xgboost.readthedocs.io/en/stable/install.html"
         msg = (
             "Cannot find XGBoost Library in the candidate path.  "
