@@ -1,6 +1,6 @@
 """Xgboost pyspark integration submodule for estimator API."""
-# pylint: disable=too-many-ancestors
-# pylint: disable=fixme, too-many-ancestors, protected-access, no-member, invalid-name
+
+# pylint: disable=fixme, protected-access, no-member, invalid-name
 # pylint: disable=unused-argument, too-many-locals
 
 import warnings
@@ -11,8 +11,8 @@ from pyspark import keyword_only
 from pyspark.ml.param import Param, Params
 from pyspark.ml.param.shared import HasProbabilityCol, HasRawPredictionCol
 
-from xgboost import XGBClassifier, XGBRanker, XGBRegressor
-
+from ..collective import Config
+from ..sklearn import XGBClassifier, XGBRanker, XGBRegressor
 from .core import (  # type: ignore
     _ClassificationModel,
     _SparkXGBEstimator,
@@ -160,6 +160,11 @@ class SparkXGBRegressor(_SparkXGBEstimator):
         Boolean value to specify if enabling sparse data optimization, if True,
         Xgboost DMatrix object will be constructed from sparse matrix instead of
         dense matrix.
+    launch_tracker_on_driver:
+        Boolean value to indicate whether the tracker should be launched on the driver side or
+        the executor side.
+    coll_cfg:
+        The collective configuration. See :py:class:`~xgboost.collective.Config`
 
     kwargs:
         A dictionary of xgboost parameters, please refer to
@@ -198,7 +203,7 @@ class SparkXGBRegressor(_SparkXGBEstimator):
     """
 
     @keyword_only
-    def __init__(
+    def __init__(  # pylint:disable=too-many-arguments
         self,
         *,
         features_col: Union[str, List[str]] = "features",
@@ -214,6 +219,8 @@ class SparkXGBRegressor(_SparkXGBEstimator):
         force_repartition: bool = False,
         repartition_random_shuffle: bool = False,
         enable_sparse_data_optim: bool = False,
+        launch_tracker_on_driver: bool = True,
+        coll_cfg: Optional[Config] = None,
         **kwargs: Any,
     ) -> None:
         super().__init__()
@@ -340,6 +347,11 @@ class SparkXGBClassifier(_SparkXGBEstimator, HasProbabilityCol, HasRawPrediction
         Boolean value to specify if enabling sparse data optimization, if True,
         Xgboost DMatrix object will be constructed from sparse matrix instead of
         dense matrix.
+    launch_tracker_on_driver:
+        Boolean value to indicate whether the tracker should be launched on the driver side or
+        the executor side.
+    coll_cfg:
+        The collective configuration. See :py:class:`~xgboost.collective.Config`
 
     kwargs:
         A dictionary of xgboost parameters, please refer to
@@ -376,7 +388,7 @@ class SparkXGBClassifier(_SparkXGBEstimator, HasProbabilityCol, HasRawPrediction
     """
 
     @keyword_only
-    def __init__(
+    def __init__(  # pylint:disable=too-many-arguments
         self,
         *,
         features_col: Union[str, List[str]] = "features",
@@ -394,6 +406,8 @@ class SparkXGBClassifier(_SparkXGBEstimator, HasProbabilityCol, HasRawPrediction
         force_repartition: bool = False,
         repartition_random_shuffle: bool = False,
         enable_sparse_data_optim: bool = False,
+        launch_tracker_on_driver: bool = True,
+        coll_cfg: Optional[Config] = None,
         **kwargs: Any,
     ) -> None:
         super().__init__()
@@ -523,6 +537,11 @@ class SparkXGBRanker(_SparkXGBEstimator):
         Boolean value to specify if enabling sparse data optimization, if True,
         Xgboost DMatrix object will be constructed from sparse matrix instead of
         dense matrix.
+    launch_tracker_on_driver:
+        Boolean value to indicate whether the tracker should be launched on the driver side or
+        the executor side.
+    coll_cfg:
+        The collective configuration. See :py:class:`~xgboost.collective.Config`
 
     kwargs:
         A dictionary of xgboost parameters, please refer to
@@ -566,7 +585,7 @@ class SparkXGBRanker(_SparkXGBEstimator):
     """
 
     @keyword_only
-    def __init__(
+    def __init__(  # pylint:disable=too-many-arguments
         self,
         *,
         features_col: Union[str, List[str]] = "features",
@@ -583,6 +602,8 @@ class SparkXGBRanker(_SparkXGBEstimator):
         force_repartition: bool = False,
         repartition_random_shuffle: bool = False,
         enable_sparse_data_optim: bool = False,
+        launch_tracker_on_driver: bool = True,
+        coll_cfg: Optional[Config] = None,
         **kwargs: Any,
     ) -> None:
         super().__init__()
