@@ -1,5 +1,5 @@
 /**
- * Copyright 2022 by XGBoost Contributors
+ * Copyright 2022-2024, XGBoost Contributors
  *
  * \brief Utilities for estimating initial score.
  */
@@ -7,18 +7,12 @@
 #ifndef XGBOOST_TREE_FIT_STUMP_H_
 #define XGBOOST_TREE_FIT_STUMP_H_
 
-#if !defined(NOMINMAX) && defined(_WIN32)
-#define NOMINMAX
-#endif  // !defined(NOMINMAX)
-
 #include <algorithm>  // std::max
 
-#include "../common/common.h"            // AssertGPUSupport
-#include "xgboost/base.h"                // GradientPair
-#include "xgboost/context.h"             // Context
-#include "xgboost/data.h"                // MetaInfo
-#include "xgboost/host_device_vector.h"  // HostDeviceVector
-#include "xgboost/linalg.h"              // TensorView
+#include "xgboost/base.h"     // GradientPair
+#include "xgboost/context.h"  // Context
+#include "xgboost/data.h"     // MetaInfo
+#include "xgboost/linalg.h"   // TensorView
 
 namespace xgboost {
 namespace tree {
