@@ -43,8 +43,8 @@ TEST(GroupData, ParallelGroupBuilder) {
   builder2.Push(2, Entry(0, 4), 0);
   builder2.Push(2, Entry(1, 5), 0);
 
-  expected_data.emplace_back(Entry(0, 4));
-  expected_data.emplace_back(Entry(1, 5));
+  expected_data.emplace_back(0, 4);
+  expected_data.emplace_back(1, 5);
   expected_offsets.emplace_back(6);
 
   EXPECT_EQ(data, expected_data);
