@@ -13,31 +13,25 @@
 
 namespace xgboost {
 #if defined(__CUDACC__)
-namespace {
+namespace detail {
 class HistogramCutsWrapper : public common::HistogramCuts {
  public:
   using SuperT = common::HistogramCuts;
-  void SetValues(std::vector<float> cuts) {
-    SuperT::cut_values_.HostVector() = std::move(cuts);
-  }
-  void SetPtrs(std::vector<uint32_t> ptrs) {
-    SuperT::cut_ptrs_.HostVector() = std::move(ptrs);
-  }
-  void SetMins(std::vector<float> mins) {
-    SuperT::min_vals_.HostVector() = std::move(mins);
-  }
+  void SetValues(std::vector<float> cuts) { SuperT::cut_values_.HostVector() = std::move(cuts); }
+  void SetPtrs(std::vector<uint32_t> ptrs) { SuperT::cut_ptrs_.HostVector() = std::move(ptrs); }
+  void SetMins(std::vector<float> mins) { SuperT::min_vals_.HostVector() = std::move(mins); }
 };
-}  //  anonymous namespace
+}  // namespace detail
 
-inline std::unique_ptr<EllpackPageImpl> BuildEllpackPage(int n_rows, int n_cols,
+inline std::unique_ptr<EllpackPageImpl> BuildEllpackPage(Context const* ctx, int n_rows, int n_cols,
                                                          bst_float sparsity = 0) {
   auto dmat = RandomDataGenerator(n_rows, n_cols, sparsity).Seed(3).GenerateDMatrix();
   const SparsePage& batch = *dmat->GetBatches<xgboost::SparsePage>().begin();
 
-  HistogramCutsWrapper cmat;
-  cmat.SetPtrs({0, 3, 6, 9, 12, 15, 18, 21, 24});
+  auto cmat = std::make_shared<detail::HistogramCutsWrapper>();
+  cmat->SetPtrs({0, 3, 6, 9, 12, 15, 18, 21, 24});
   // 24 cut fields, 3 cut fields for each feature (column).
-  cmat.SetValues({0.30f, 0.67f, 1.64f,
+  cmat->SetValues({0.30f, 0.67f, 1.64f,
           0.32f, 0.77f, 1.95f,
           0.29f, 0.70f, 1.80f,
           0.32f, 0.75f, 1.85f,
@@ -45,7 +39,7 @@ inline std::unique_ptr<EllpackPageImpl> BuildEllpackPage(int n_rows, int n_cols,
           0.25f, 0.74f, 2.00f,
           0.26f, 0.74f, 1.98f,
           0.26f, 0.71f, 1.83f});
-  cmat.SetMins({0.1f, 0.2f, 0.3f, 0.1f, 0.2f, 0.3f, 0.2f, 0.2f});
+  cmat->SetMins({0.1f, 0.2f, 0.3f, 0.1f, 0.2f, 0.3f, 0.2f, 0.2f});
 
   bst_idx_t row_stride = 0;
   const auto &offset_vec = batch.offset.ConstHostVector();
@@ -54,7 +48,7 @@ inline std::unique_ptr<EllpackPageImpl> BuildEllpackPage(int n_rows, int n_cols,
   }
 
   auto page = std::unique_ptr<EllpackPageImpl>(
-      new EllpackPageImpl(DeviceOrd::CUDA(0), cmat, batch, dmat->IsDense(), row_stride, {}));
+      new EllpackPageImpl(ctx, cmat, batch, dmat->IsDense(), row_stride, {}));
 
   return page;
 }
