@@ -1,28 +1,28 @@
 /**
- * Copyright 2023 by XGBoost Contributors
+ * Copyright 2023-2025, XGBoost Contributors
  */
 #include "test_lambdarank_obj.h"
 
-#include <gtest/gtest.h>                        // for Test, Message, TestPartResult, CmpHel...
+#include <gtest/gtest.h>  // for Test, Message, TestPartResult, CmpHel...
 
-#include <algorithm>                            // for sort
-#include <cstddef>                              // for size_t
-#include <initializer_list>                     // for initializer_list
-#include <map>                                  // for map
-#include <memory>                               // for unique_ptr, shared_ptr, make_shared
-#include <numeric>                              // for iota
-#include <string>                               // for char_traits, basic_string, string
-#include <vector>                               // for vector
+#include <algorithm>         // for sort
+#include <cstddef>           // for size_t
+#include <initializer_list>  // for initializer_list
+#include <memory>            // for unique_ptr, shared_ptr, make_shared
+#include <numeric>           // for iota
+#include <string>            // for char_traits, basic_string, string
+#include <vector>            // for vector
 
-#include "../../../src/common/ranking_utils.h"  // for NDCGCache, LambdaRankParam
-#include "../helpers.h"                         // for CheckRankingObjFunction, CheckConfigReload
-#include "xgboost/base.h"                       // for GradientPair, bst_group_t, Args
-#include "xgboost/context.h"                    // for Context
-#include "xgboost/data.h"                       // for MetaInfo, DMatrix
-#include "xgboost/host_device_vector.h"         // for HostDeviceVector
-#include "xgboost/linalg.h"                     // for Tensor, All, TensorView
-#include "xgboost/objective.h"                  // for ObjFunction
-#include "xgboost/span.h"                       // for Span
+#include "../../../src/common/ranking_utils.h"      // for NDCGCache, LambdaRankParam
+#include "../../../src/objective/lambdarank_obj.h"  // for MAPStat, MakePairs
+#include "../helpers.h"                  // for CheckRankingObjFunction, CheckConfigReload
+#include "xgboost/base.h"                // for GradientPair, bst_group_t, Args
+#include "xgboost/context.h"             // for Context
+#include "xgboost/data.h"                // for MetaInfo, DMatrix
+#include "xgboost/host_device_vector.h"  // for HostDeviceVector
+#include "xgboost/linalg.h"              // for Tensor, All, TensorView
+#include "xgboost/objective.h"           // for ObjFunction
+#include "xgboost/span.h"                // for Span
 
 namespace xgboost::obj {
 TEST(LambdaRank, NDCGJsonIO) {
@@ -106,6 +106,20 @@ void TestNDCGGPair(Context const* ctx) {
     }
   }
 
+  {
+    // Test empty input
+    std::unique_ptr<xgboost::ObjFunction> obj{xgboost::ObjFunction::Create("rank:ndcg", ctx)};
+    obj->Configure(Args{{"lambdarank_pair_method", "topk"}});
+
+    HostDeviceVector<float> predts;
+    MetaInfo info;
+    info.labels = linalg::Tensor<float, 2>{{}, {0, 1}, ctx->Device()};
+    info.group_ptr_ = {0, 0};
+    info.num_row_ = 0;
+    linalg::Matrix<GradientPair> gpairs;
+    obj->GetGradient(predts, info, 0, &gpairs);
+    ASSERT_EQ(gpairs.Size(), 0);
+  }
   ASSERT_NO_THROW(obj->DefaultEvalMetric());
 }
 
@@ -119,7 +133,8 @@ void TestUnbiasedNDCG(Context const* ctx) {
   obj->Configure(Args{{"lambdarank_pair_method", "topk"},
                       {"lambdarank_unbiased", "true"},
                       {"lambdarank_bias_norm", "0"}});
-  std::shared_ptr<DMatrix> p_fmat{RandomDataGenerator{10, 1, 0.0f}.GenerateDMatrix(true, false, 2)};
+  std::shared_ptr<DMatrix> p_fmat{
+      RandomDataGenerator{10, 1, 0.0f}.Classes(2).GenerateDMatrix(true)};
   auto h_label = p_fmat->Info().labels.HostView().Values();
   // Move clicked samples to the beginning.
   std::sort(h_label.begin(), h_label.end(), std::greater<>{});
@@ -245,9 +260,9 @@ void TestMAPStat(Context const* ctx) {
 
     predt.SetDevice(ctx->Device());
     auto rank_idx =
-        p_cache->SortedIdx(ctx, ctx->IsCPU() ? predt.ConstHostSpan() : predt.ConstDeviceSpan());
+        p_cache->SortedIdx(ctx, !ctx->IsCUDA() ? predt.ConstHostSpan() : predt.ConstDeviceSpan());
 
-    if (ctx->IsCPU()) {
+    if (!ctx->IsCUDA()) {
       obj::cpu_impl::MAPStat(ctx, info.labels.HostView().Slice(linalg::All(), 0), rank_idx,
                              p_cache);
     } else {
@@ -282,9 +297,9 @@ void TestMAPStat(Context const* ctx) {
 
     predt.SetDevice(ctx->Device());
     auto rank_idx =
-        p_cache->SortedIdx(ctx, ctx->IsCPU() ? predt.ConstHostSpan() : predt.ConstDeviceSpan());
+        p_cache->SortedIdx(ctx, !ctx->IsCUDA() ? predt.ConstHostSpan() : predt.ConstDeviceSpan());
 
-    if (ctx->IsCPU()) {
+    if (!ctx->IsCUDA()) {
       obj::cpu_impl::MAPStat(ctx, info.labels.HostView().Slice(linalg::All(), 0), rank_idx,
                              p_cache);
     } else {
