@@ -89,8 +89,7 @@ void TestBasic(DMatrix* dmat, Context const * ctx);
 
 // p_full and p_hist should come from the same data set.
 void TestTrainingPrediction(Context const* ctx, size_t rows, size_t bins,
-                            std::shared_ptr<DMatrix> p_full, std::shared_ptr<DMatrix> p_hist,
-                            bool check_contribs = false);
+                            std::shared_ptr<DMatrix> p_full, std::shared_ptr<DMatrix> p_hist);
 
 void TestInplacePrediction(Context const* ctx, std::shared_ptr<DMatrix> x, bst_idx_t rows,
                            bst_feature_t cols);
@@ -114,6 +113,11 @@ void TestSparsePrediction(Context const* ctx, float sparsity);
 void TestSparsePredictionColumnSplit(int world_size, bool use_gpu, float sparsity);
 
 void TestVectorLeafPrediction(Context const* ctx);
+
+class ShapExternalMemoryTest : public ::testing::TestWithParam<std::tuple<bool, bool>> {
+ public:
+  void Run(Context const* ctx, bool is_qdm, bool is_interaction);
+};
 }  // namespace xgboost
 
 #endif  // XGBOOST_TEST_PREDICTOR_H_
