@@ -1,15 +1,16 @@
-// Copyright by Contributors
+/**
+ * Copyright 2016-2024, XGBoost Contributors
+ */
 #include <gtest/gtest.h>
 #include <xgboost/base.h>
 #include <xgboost/logging.h>
+
 #include <string>
-#include <memory>
-#include <vector>
 
 #include "helpers.h"
 
-int main(int argc, char ** argv) {
-  xgboost::Args args {{"verbosity", "2"}};
+int main(int argc, char** argv) {
+  xgboost::Args args{{"verbosity", "2"}};
   xgboost::ConsoleLogger::Configure(args);
 
   testing::InitGoogleTest(&argc, argv);
