@@ -78,10 +78,18 @@ void CompareJSON(Json l, Json r) {
     }
     break;
   }
+  case Value::ValueKind::kI8Array: {
+    CompareIntArray<I8Array>(l, r);
+    break;
+  }
   case Value::ValueKind::kU8Array: {
     CompareIntArray<U8Array>(l, r);
     break;
   }
+  case Value::ValueKind::kI16Array: {
+    CompareIntArray<I16Array>(l, r);
+    break;
+  }
   case Value::ValueKind::kI32Array: {
     CompareIntArray<I32Array>(l, r);
     break;
