@@ -4,18 +4,11 @@ import random
 import tempfile
 import uuid
 from collections import namedtuple
-from typing import Generator, Sequence, Type
+from typing import Generator, Iterable, List, Sequence
 
 import numpy as np
 import pytest
 from pyspark import SparkConf
-
-import xgboost as xgb
-from xgboost import testing as tm
-from xgboost.spark.data import pred_contribs
-
-pytestmark = [tm.timeout(60), pytest.mark.skipif(**tm.no_spark())]
-
 from pyspark.ml import Pipeline, PipelineModel
 from pyspark.ml.evaluation import BinaryClassificationEvaluator
 from pyspark.ml.feature import VectorAssembler
@@ -25,7 +18,10 @@ from pyspark.ml.tuning import CrossValidator, ParamGridBuilder
 from pyspark.sql import SparkSession
 from pyspark.sql import functions as spark_sql_func
 
+import xgboost as xgb
 from xgboost import XGBClassifier, XGBModel, XGBRegressor
+from xgboost import testing as tm
+from xgboost.collective import Config
 from xgboost.spark import (
     SparkXGBClassifier,
     SparkXGBClassifierModel,
@@ -34,11 +30,14 @@ from xgboost.spark import (
     SparkXGBRegressorModel,
 )
 from xgboost.spark.core import _non_booster_params
+from xgboost.spark.data import pred_contribs
 
 from .utils import SparkTestCase
 
 logging.getLogger("py4j").setLevel(logging.INFO)
 
+pytestmark = [tm.timeout(60), pytest.mark.skipif(**tm.no_spark())]
+
 
 def no_sparse_unwrap() -> tm.PytestSkip:
     try:
@@ -80,6 +79,8 @@ RegWithWeight = namedtuple(
         "reg_df_test_with_eval_weight",
         "reg_with_eval_best_score",
         "reg_with_eval_and_weight_best_score",
+        "reg_expected_evals_result_train",
+        "reg_expected_evals_result_validation",
     ),
 )
 
@@ -129,6 +130,14 @@ def reg_with_weight(
     predt3 = reg3.predict(X)
     best_score3 = reg3.best_score
 
+    reg4 = XGBRegressor(eval_metric="rmse")
+    reg4.fit(X_train, y_train, eval_set=[(X_train, y_train), (X_val, y_val)])
+    reg_expected_evals_result = reg4.evals_result()
+    reg_expected_evals_result_train = reg_expected_evals_result["validation_0"]["rmse"]
+    reg_expected_evals_result_validation = reg_expected_evals_result["validation_1"][
+        "rmse"
+    ]
+
     reg_df_train_with_eval_weight = spark.createDataFrame(
         [
             (Vectors.dense(1.0, 2.0, 3.0), 0, False, 1.0),
@@ -167,6 +176,8 @@ def reg_with_weight(
         reg_df_test_with_eval_weight,
         best_score2,
         best_score3,
+        reg_expected_evals_result_train,
+        reg_expected_evals_result_validation,
     )
 
 
@@ -289,6 +300,8 @@ ClfWithWeight = namedtuple(
         "cls_df_test_with_eval_weight",
         "cls_with_eval_best_score",
         "cls_with_eval_and_weight_best_score",
+        "cls_expected_evals_result_train",
+        "cls_expected_evals_result_validation",
     ),
 )
 
@@ -327,6 +340,16 @@ def clf_with_weight(
         sample_weight_eval_set=[w_val],
     )
 
+    cls4 = XGBClassifier(eval_metric="logloss")
+    cls4.fit(
+        X_train,
+        y_train,
+        eval_set=[(X_train, y_train), (X_val, y_val)],
+    )
+    cls4_evals_result = cls4.evals_result()
+    cls_expected_evals_result_train = cls4_evals_result["validation_0"]["logloss"]
+    cls_expected_evals_result_validation = cls4_evals_result["validation_1"]["logloss"]
+
     cls_df_train_with_eval_weight = spark.createDataFrame(
         [
             (Vectors.dense(1.0, 2.0, 3.0), 0, False, 1.0),
@@ -365,6 +388,8 @@ def clf_with_weight(
         cls_df_test_with_eval_weight,
         cls_with_eval_best_score,
         cls_with_eval_and_weight_best_score,
+        cls_expected_evals_result_train,
+        cls_expected_evals_result_validation,
     )
 
 
@@ -1188,6 +1213,100 @@ class TestPySparkLocal:
                         "3.5.1", conf
                     )
 
+    @pytest.mark.parametrize("tree_method", ["hist", "approx"])
+    def test_empty_train_data(self, spark: SparkSession, tree_method: str) -> None:
+        df_train = spark.createDataFrame(
+            [
+                (Vectors.dense(10.1, 11.2, 11.3), 0, True),
+                (Vectors.dense(1, 1.2, 1.3), 1, True),
+                (Vectors.dense(14.0, 15.0, 16.0), 0, True),
+                (Vectors.dense(1.1, 1.2, 1.3), 1, False),
+            ],
+            ["features", "label", "val_col"],
+        )
+        classifier = SparkXGBRegressor(
+            num_workers=2,
+            min_child_weight=0.0,
+            reg_alpha=0,
+            reg_lambda=0,
+            tree_method=tree_method,
+            validation_indicator_col="val_col",
+        )
+        model = classifier.fit(df_train)
+        pred_result = model.transform(df_train).collect()
+        for row in pred_result:
+            assert row.prediction == 1.0
+
+    def test_regressor_xgb_summary(self, reg_with_weight: RegWithWeight) -> None:
+        reg_df_train = reg_with_weight.reg_df_train_with_eval_weight.filter(
+            spark_sql_func.col("isVal") == False
+        )
+        spark_xgb_model = SparkXGBRegressor(eval_metric="rmse").fit(reg_df_train)
+
+        np.testing.assert_allclose(
+            reg_with_weight.reg_expected_evals_result_train,
+            spark_xgb_model.training_summary.train_objective_history["rmse"],
+            atol=1e-3,
+        )
+
+        assert spark_xgb_model.training_summary.validation_objective_history == {}
+
+    def test_regressor_xgb_summary_with_validation(
+        self, reg_with_weight: RegWithWeight
+    ) -> None:
+        spark_xgb_model = SparkXGBRegressor(
+            eval_metric="rmse", validation_indicator_col="isVal"
+        ).fit(
+            reg_with_weight.reg_df_train_with_eval_weight,
+        )
+
+        np.testing.assert_allclose(
+            reg_with_weight.reg_expected_evals_result_train,
+            spark_xgb_model.training_summary.train_objective_history["rmse"],
+            atol=1e-3,
+        )
+
+        np.testing.assert_allclose(
+            reg_with_weight.reg_expected_evals_result_validation,
+            spark_xgb_model.training_summary.validation_objective_history["rmse"],
+            atol=1e-3,
+        )
+
+    def test_classifier_xgb_summary(self, clf_with_weight: ClfWithWeight) -> None:
+        clf_df_train = clf_with_weight.cls_df_train_with_eval_weight.filter(
+            spark_sql_func.col("isVal") == False
+        )
+        spark_xgb_model = SparkXGBClassifier(eval_metric="logloss").fit(clf_df_train)
+
+        np.testing.assert_allclose(
+            clf_with_weight.cls_expected_evals_result_train,
+            spark_xgb_model.training_summary.train_objective_history["logloss"],
+            atol=1e-3,
+        )
+
+        assert spark_xgb_model.training_summary.validation_objective_history == {}
+
+    def test_classifier_xgb_summary_with_validation(
+        self, clf_with_weight: ClfWithWeight
+    ) -> None:
+        spark_xgb_model = SparkXGBClassifier(
+            eval_metric="logloss", validation_indicator_col="isVal"
+        ).fit(
+            clf_with_weight.cls_df_train_with_eval_weight,
+        )
+
+        np.testing.assert_allclose(
+            clf_with_weight.cls_expected_evals_result_train,
+            spark_xgb_model.training_summary.train_objective_history["logloss"],
+            atol=1e-3,
+        )
+
+        np.testing.assert_allclose(
+            clf_with_weight.cls_expected_evals_result_validation,
+            spark_xgb_model.training_summary.validation_objective_history["logloss"],
+            atol=1e-3,
+        )
+
 
 class XgboostLocalTest(SparkTestCase):
     def setUp(self):
@@ -1599,33 +1718,6 @@ class XgboostLocalTest(SparkTestCase):
             for row in pred_result:
                 self.assertEqual(row.prediction, row.label)
 
-    def test_empty_train_data(self) -> None:
-        for tree_method in [
-            "hist",
-            "approx",
-        ]:  # pytest.mark conflict with python unittest
-            df_train = self.session.createDataFrame(
-                [
-                    (Vectors.dense(10.1, 11.2, 11.3), 0, True),
-                    (Vectors.dense(1, 1.2, 1.3), 1, True),
-                    (Vectors.dense(14.0, 15.0, 16.0), 0, True),
-                    (Vectors.dense(1.1, 1.2, 1.3), 1, False),
-                ],
-                ["features", "label", "val_col"],
-            )
-            classifier = SparkXGBClassifier(
-                num_workers=2,
-                min_child_weight=0.0,
-                reg_alpha=0,
-                reg_lambda=0,
-                tree_method=tree_method,
-                validation_indicator_col="val_col",
-            )
-            model = classifier.fit(df_train)
-            pred_result = model.transform(df_train).collect()
-            for row in pred_result:
-                assert row.prediction == 1.0
-
     def test_empty_partition(self):
         # raw_df.repartition(4) will result int severe data skew, actually,
         # there is no any data in reducer partition 1, reducer partition 2
@@ -1650,8 +1742,89 @@ class XgboostLocalTest(SparkTestCase):
         with pytest.raises(ValueError, match="evals_result"):
             SparkXGBClassifier(evals_result={})
 
+    def test_collective_conf(self):
+        classifier = SparkXGBClassifier(
+            launch_tracker_on_driver=True,
+            coll_cfg=Config(tracker_host_ip="192.168.1.32", tracker_port=59981),
+        )
+        with pytest.raises(Exception, match="Failed to bind socket"):
+            classifier._get_tracker_args()
 
-LTRData = namedtuple("LTRData", ("df_train", "df_test", "df_train_1"))
+        classifier = SparkXGBClassifier(
+            launch_tracker_on_driver=False,
+            coll_cfg=Config(tracker_host_ip="127.0.0.1", tracker_port=58892),
+        )
+        with pytest.raises(
+            ValueError, match="You must enable launch_tracker_on_driver"
+        ):
+            classifier._get_tracker_args()
+
+        classifier = SparkXGBClassifier(
+            launch_tracker_on_driver=True,
+            coll_cfg=Config(tracker_host_ip="127.0.0.1", tracker_port=58893),
+            num_workers=2,
+        )
+        launch_tracker_on_driver, rabit_envs = classifier._get_tracker_args()
+        assert launch_tracker_on_driver is True
+        assert rabit_envs["n_workers"] == 2
+        assert rabit_envs["dmlc_tracker_uri"] == "127.0.0.1"
+        assert rabit_envs["dmlc_tracker_port"] == 58893
+
+        with tempfile.TemporaryDirectory() as tmpdir:
+            path = "file:" + tmpdir
+            classifier = SparkXGBClassifier(
+                launch_tracker_on_driver=True,
+                coll_cfg=Config(tracker_host_ip="127.0.0.1", tracker_port=58894),
+                num_workers=1,
+                n_estimators=1,
+            )
+
+            def check_conf(conf: Config) -> None:
+                assert conf.tracker_host_ip == "127.0.0.1"
+                assert conf.tracker_port == 58894
+
+            check_conf(classifier.getOrDefault(classifier.coll_cfg))
+            classifier.write().overwrite().save(path)
+
+            loaded_classifier = SparkXGBClassifier.load(path)
+            check_conf(loaded_classifier.getOrDefault(loaded_classifier.coll_cfg))
+
+            model = classifier.fit(self.cls_df_sparse_train)
+            check_conf(model.getOrDefault(model.coll_cfg))
+
+            model.write().overwrite().save(path)
+            loaded_model = SparkXGBClassifierModel.load(path)
+            check_conf(loaded_model.getOrDefault(loaded_model.coll_cfg))
+
+    def test_classifier_with_multi_cols(self):
+        df = self.session.createDataFrame(
+            [
+                (1.0, 2.0, 0),
+                (3.1, 4.2, 1),
+            ],
+            ["a", "b", "label"],
+        )
+        features = ["a", "b"]
+        cls = SparkXGBClassifier(features_col=features, device="cpu", n_estimators=2)
+        model = cls.fit(df)
+        self.assertEqual(features, model.getOrDefault(model.features_cols))
+        self.assertTrue(not model.isSet(model.featuresCol))
+
+        # No exception
+        model.transform(df).collect()
+
+
+LTRData = namedtuple(
+    "LTRData",
+    (
+        "df_train",
+        "df_test",
+        "df_train_1",
+        "ranker_df_merged",
+        "expected_evals_result_train",
+        "expected_evals_result_validation",
+    ),
+)
 
 
 @pytest.fixture
@@ -1691,22 +1864,47 @@ def ltr_data(spark: SparkSession) -> Generator[LTRData, None, None]:
             [np.nan, 8.0, 10.5],
         ]
     )
+    qid_test = np.array([0, 0, 0, 1, 1, 1])
+    y_test = np.array([1, 0, 2, 1, 1, 2])
 
     ltr = xgb.XGBRanker(tree_method="approx", objective="rank:pairwise")
     ltr.fit(X_train, y_train, qid=qid_train)
     predt = ltr.predict(X_test)
 
+    ltr2 = xgb.XGBRanker(tree_method="approx", objective="rank:pairwise")
+    ltr2.fit(
+        X_train,
+        y_train,
+        qid=qid_train,
+        eval_set=[(X_train, y_train), (X_test, y_test)],
+        eval_qid=[qid_train, qid_test],
+    )
+    evals_result = ltr2.evals_result()
+    expected_evals_result_train = evals_result["validation_0"]["ndcg@32"]
+    expected_evals_result_validation = evals_result["validation_1"]["ndcg@32"]
+
     ranker_df_test = spark.createDataFrame(
         [
-            (Vectors.dense(1.5, 2.0, 3.0), 0, float(predt[0])),
-            (Vectors.dense(4.5, 5.0, 6.0), 0, float(predt[1])),
-            (Vectors.dense(9.0, 4.5, 8.0), 0, float(predt[2])),
-            (Vectors.sparse(3, {1: 1.0, 2: 6.0}), 1, float(predt[3])),
-            (Vectors.sparse(3, {1: 6.0, 2: 7.0}), 1, float(predt[4])),
-            (Vectors.sparse(3, {1: 8.0, 2: 10.5}), 1, float(predt[5])),
+            (Vectors.dense(1.5, 2.0, 3.0), 0, float(predt[0]), 1),
+            (Vectors.dense(4.5, 5.0, 6.0), 0, float(predt[1]), 0),
+            (Vectors.dense(9.0, 4.5, 8.0), 0, float(predt[2]), 2),
+            (Vectors.sparse(3, {1: 1.0, 2: 6.0}), 1, float(predt[3]), 1),
+            (Vectors.sparse(3, {1: 6.0, 2: 7.0}), 1, float(predt[4]), 1),
+            (Vectors.sparse(3, {1: 8.0, 2: 10.5}), 1, float(predt[5]), 2),
         ],
-        ["features", "qid", "expected_prediction"],
+        ["features", "qid", "expected_prediction", "label"],
     )
+
+    ranker_df_merged = (
+        ranker_df_train.select(["features", "label", "qid"])
+        .withColumn("isVal", spark_sql_func.lit(False))
+        .union(
+            ranker_df_test.select(["features", "label", "qid"]).withColumn(
+                "isVal", spark_sql_func.lit(True)
+            )
+        )
+    )
+
     ranker_df_train_1 = spark.createDataFrame(
         [
             (Vectors.sparse(3, {1: 1.0, 2: 5.5}), 0, 9),
@@ -1725,7 +1923,14 @@ def ltr_data(spark: SparkSession) -> Generator[LTRData, None, None]:
         * 4,
         ["features", "label", "qid"],
     )
-    yield LTRData(ranker_df_train, ranker_df_test, ranker_df_train_1)
+    yield LTRData(
+        ranker_df_train,
+        ranker_df_test,
+        ranker_df_train_1,
+        ranker_df_merged,
+        expected_evals_result_train,
+        expected_evals_result_validation,
+    )
 
 
 class TestPySparkLocalLETOR:
@@ -1742,3 +1947,49 @@ class TestPySparkLocalLETOR:
         assert ranker.getOrDefault(ranker.objective) == "rank:ndcg"
         model = ranker.fit(ltr_data.df_train_1)
         model.transform(ltr_data.df_test).collect()
+
+    def test_ranker_same_qid_in_same_partition(self, ltr_data: LTRData) -> None:
+        ranker = SparkXGBRanker(qid_col="qid", num_workers=4, force_repartition=True)
+        df, _ = ranker._prepare_input(ltr_data.df_train_1)
+
+        def f(iterator: Iterable) -> List[int]:
+            yield list(set(iterator))
+
+        rows = df.select("qid").rdd.mapPartitions(f).collect()
+        assert len(rows) == 4
+        for row in rows:
+            assert len(row) == 1
+            assert row[0].qid in [6, 7, 8, 9]
+
+    def test_ranker_xgb_summary(self, ltr_data: LTRData) -> None:
+        spark_xgb_model = SparkXGBRanker(
+            tree_method="approx", qid_col="qid", objective="rank:pairwise"
+        ).fit(ltr_data.df_train)
+
+        np.testing.assert_allclose(
+            ltr_data.expected_evals_result_train,
+            spark_xgb_model.training_summary.train_objective_history["ndcg@32"],
+            atol=1e-3,
+        )
+
+        assert spark_xgb_model.training_summary.validation_objective_history == {}
+
+    def test_ranker_xgb_summary_with_validation(self, ltr_data: LTRData) -> None:
+        spark_xgb_model = SparkXGBRanker(
+            tree_method="approx",
+            qid_col="qid",
+            objective="rank:pairwise",
+            validation_indicator_col="isVal",
+        ).fit(ltr_data.ranker_df_merged)
+
+        np.testing.assert_allclose(
+            ltr_data.expected_evals_result_train,
+            spark_xgb_model.training_summary.train_objective_history["ndcg@32"],
+            atol=1e-3,
+        )
+
+        np.testing.assert_allclose(
+            ltr_data.expected_evals_result_validation,
+            spark_xgb_model.training_summary.validation_objective_history["ndcg@32"],
+            atol=1e-3,
+        )
